<?php

namespace Modules\Cadastros\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Modules\Cadastros\Services\Cmp02600ParticularidadesService;

class ParticularidadesController extends \Teknisa\GER\Controllers\TeknisaCrud
{
    const MESSAGES = [
        '1' => "MANDATORY_MARK",
        '2' => "PREFERRED_BRAND",
        '3' => "RESTRICTION_BRAND",
        '4' => "MANDATORY_SUPPLIER",
        '5' => "PREFERRED_SUPPLIER",
        '6' => "RESTRICTION_SUPPLIER",
        '7' => "MANDATORY_BRAND_SUPPLIER",
        '8' => "PREFERRED_BRAND_SUPPLIER",
        '9' => "REGISTERED_BRAND_SUPPLIER",
    ];
    const TIPOS = [
        '1' => "Marca Obrigatória",
        '2' => "Marca Preferencial",
        '3' => "Restrição à Marca",
        '4' => "Fornecedor Obrigatório",
        '5' => "Fornecedor Preferencial",
        '6' => "Restrição ao Fornecedor",
        '7' => "Marca/Fornecedor Obrigatórios",
        '8' => "Marca/Fornecedor Preferenciais",
        '9' => "Restrição à Marca/Fornecedor"
    ];
    const MESSAGESEXPORT = [
        '1' => "PARTICULARITY_MANDATORY_BRAND",
        '2' => "PARTICULARITY_PREFERRED_BRAND",
        '3' => "PARTICULARITY_RESTRICTION_BRAND",
        '4' => "PARTICULARITY_MANDATORY_SUPPLIER",
        '5' => "PARTICULARITY_PREFERRED_SUPPLIER",
        '6' => "PARTICULARITY_RESTRICTION_SUPPLIER",
        '7' => "PARTICULARITY_MANDATORY_BRAND_SUPPLIER",
        '8' => "PARTICULARITY_PREFERRED_BRAND_SUPPLIER",
        '9' => "PARTICULARITY_REGISTERED_BRAND_SUPPLIER",
    ];
    const MESSAGESEXPORTSAMETYPE = [
        '1' => "PARTICULARITY_MANDATORY_BRAND_OTHER",
        '2' => "PARTICULARITY_PREFERRED_BRAND_OTHER",
        '3' => "PARTICULARITY_RESTRICTION_BRAND_OTHER",
        '4' => "PARTICULARITY_MANDATORY_SUPPLIER_OTHER",
        '5' => "PARTICULARITY_PREFERRED_SUPPLIER_OTHER",
        '6' => "PARTICULARITY_RESTRICTION_SUPPLIER_OTHER",
        '7' => "PARTICULARITY_MANDATORY_BRAND_SUPPLIER_OTHER",
        '8' => "PARTICULARITY_PREFERRED_BRAND_SUPPLIER_OTHER",
        '9' => "PARTICULARITY_REGISTERED_BRAND_SUPPLIER_OTHER",
    ];

    protected $environment;
    protected $stNomeRel;
    protected $stExtensao;
    protected $parameterBag;

    public function __construct($crudService, ParameterBag $parameterBag, Cmp02600ParticularidadesService $service, $environment)
    {
        parent::__construct($crudService, $parameterBag);
        $this->parameterBag = $parameterBag;
        $this->service = $service;
        $this->environment = $environment;
    }

    private function createFilterCriteria($params)
    {
        $filterCriteria = new FilterCriteria($this->dataSourceName);

        foreach ($params as $name => $value) {
            $filterCriteria->addCondition($name, '=', $value);
        }

        return $filterCriteria;
    }

    public function generateCode($row, $request)
    {
        try {
            $this->dataSourceName = 'nextNrSeqPart';
            $params = array(
                'CDFILIAL'  => $row['CDFILIAL'],
                'CDPRODUTO' => $row['CDPRODUTO']
            );
            $filterCriteria = $this->createFilterCriteria($params);
            $newRequest = new DTO\Request\Filter($filterCriteria, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
            $newResponse = new DTO\Response();

            parent::find($newRequest, $newResponse);
            $data = $newResponse->getDataSets();

            if (is_array($data) && count($data)) {
                $rows = end($data)->getRows();
                if (is_array($rows) && count($rows)) {
                    return end($rows)['NRSEQPART'];
                }
            }

            return 1;
        } catch (Exception $e) {
            throw $e;
        }
    }

    public function addNewProduct(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            $particularidade  = $row['CDTIPPART'];
            $newProduct = true;

            for ($i = 0; $i < count($particularidade); $i++) {
                $secondRow['CDPRODUTO']  = $row['CDPRODUTO'];
                $secondRow['CDARVPROD']  = $row['CDARVPROD'];
                $secondRow['CDFILIAL']   = $row['CDFILIAL'];
                $secondRow['NRSEQPART']  = self::generateCode($row, $request);
                $secondRow['CDTIPPART']  = $particularidade[$i];
                $secondRow['CDMARCA']    = $row['CDMARCA'];
                $secondRow['CDFORNECED'] = $row['CDFORNECED'];
                $secondRow['CDNVPRODUTO'] = $row['CDNVPRODUTO'];

                if ($particularidade[$i] <= 3) {
                    $secondRow['CDFORNECED'] = null;
                } else if ($particularidade[$i] <= 6) {
                    $secondRow['CDMARCA'] = null;
                }
                if ($newProduct == true) {
                    $newProduct = false;
                    self::validaInclusao($secondRow, $response);
                } else {
                    if ($particularidade[$i] <= 3) {
                        //self::validateParticularity($this->service->getExistingBrandParticularities($secondRow));
                        self::brandValidate($secondRow);
                    } else if ($particularidade[$i] <= 6) {
                        //self::validateParticularity($this->service->getExistingProviderParticularities($secondRow));
                        self::providerValidate($secondRow);
                    } else {
                        //self::validateParticularity($this->service->getExistingBrandProviderParticularities($secondRow));
                        self::brandProviderValidate($secondRow);
                    }
                }
                $this->service->insertNewProductOrParticularity($secondRow);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }

    public function validaInclusao($row, $response)
    {
        if (!empty($this->service->validaProdutoExiste($row))) {
            throw new \Exception("ParticularityRegisteredInformedProduct");
        } elseif ($row['CDMARCA'] && empty($this->service->validaMarcaProduto($row))) {
            throw new \Exception("BrandNotRegisteredInformedProduct");
        }

        return true;
    }

    public function deleteProduct(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();

            $products = $this->service->getProductInformation($row);

            foreach ($products as $product) {
                $this->service->deleteProductOrParticularity($product);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }

    public function deleteProductOrParticularity(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();

            $this->service->deleteProductOrParticularity($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }

    public function addNewParticularidadePorMarca(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();

            //self::validateParticularity($this->service->getExistingBrandParticularities($row));
            self::brandValidate($row);

            $row['NRSEQPART']  = self::generateCode($row, $request);
            $row['CDFORNECED'] = null;
            $this->service->insertNewProductOrParticularity($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }

    public function addNewParticularidadePorFornecedor(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            //self::validateParticularity($this->service->getExistingProviderParticularities($row));

            self::providerValidate($row);

            $row['NRSEQPART']  = self::generateCode($row, $request);
            $row['CDMARCA'] = null;

            $this->service->insertNewProductOrParticularity($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }

    public function addNewParticularidadePorMarcaFornecedor(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();

            //self::validateParticularity($this->service->getExistingBrandProviderParticularities($row));

            self::brandProviderValidate($row);

            $row['NRSEQPART']  = self::generateCode($row, $request);

            $this->service->insertNewProductOrParticularity($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }

    public function validateParticularity($particularities)
    {
        if (!empty($particularities)) {
            $cdParticularity = $particularities[0]['CDTIPPART'];

            throw new \Exception(self::MESSAGES[$cdParticularity]);
        }
    }

    public function brandValidate($row)
    {
        if (empty($this->service->validaMarcaProduto($row))) {
            throw new \Exception("PesqMarcaProd");
        }

        if (!empty($this->service->validateBrandExists($row))) {
            throw new \Exception("BrandReadyRegistered");
        }
    }

    public function providerValidate($row)
    {

        if (!empty($this->service->validateProviderExists($row))) {
            throw new \Exception("FORN_JA_CADASTRADO");
        }
    }

    public function brandProviderValidate($row)
    {

        if (empty($this->service->validaMarcaProduto($row))) {
            throw new \Exception("PesqMarcaProd");
        }

        if (!empty($this->service->validateBrandProviderExists($row))) {
            throw new \Exception("SupplierBrandRegistered");
        }
    }

    public function updateBrandParticularity(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            if ($row['CDMARCA'] != $row['OLD_CDMARCA']) {
                self::brandValidate($row);
            }

            $row['CDFORNECED'] = null;

            $this->service->updateParticularity($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }

    public function updateProviderParticularity(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();

            if ($row['CDFORNECED'] != $row['OLD_CDFORNECED']) {
                self::providerValidate($row);
            }

            $row['CDMARCA'] = null;

            $this->service->updateParticularity($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }

    public function updateBrandProviderParticularity(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();

            if ($row['CDMARCA'] != $row['OLD_CDMARCA'] || $row['CDFORNECED'] != $row['OLD_CDFORNECED']) {
                self::brandProviderValidate($row);
            }

            $this->service->updateParticularity($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }

    public function autoDeleteParticularities(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();

            self::prepareRowToProcedure($row);

            $this->service->procedureExcluirPartFili($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }

    public function prepareRowToProcedure($row)
    {
        $row['CDFORNECED'] = $row['CDFORNECED'] == "T" ? null : $row['CDFORNECED'];
        $row['CDMARCA']    = $row['CDMARCA']    == "T" ? null : $row['CDMARCA'];
        $row['CDFILIAL']   = $row['CDFILIAL']   == "T" ? null : $row['CDFILIAL'];
        $row['TPRODUTO']   = $row['TPRODUTO']   != "T" ? " "  : $row['TPRODUTO'];
    }

    public function exportParticulatiries(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            $erros = array();
            $produtosExcluidos = array();
            $filiais = self::buscaFiliais($row);
            $particularidades = self::buscaParticularidades($row);

            foreach ($filiais as $filial) {
                if ($row['IDACAOEXPORT'] == 'C') {
                    $this->service->deleteClonar($filial, $row);
                }
                foreach ($particularidades as $particularidade) {
                    $novaParticularidade = self::arrayNovaParticularidade($particularidade, $filial, $request);
                    $produto = self::arrayProdutoExcluido($particularidade, $filial);

                    if ($row['IDACAOEXPORT'] == 'S' and !in_array($produto, $produtosExcluidos)) {
                        $this->service->deleteSubstituir($filial, $particularidade);
                        array_push($produtosExcluidos, $produto);
                    }
                    if ($novaParticularidade['CDTIPPART'] <= 3) {
                        $validaParticularidade = $this->service->getExistingBrandParticularities($novaParticularidade);
                        if (empty($this->service->validaMarcaProduto($novaParticularidade))) {
                            array_push($erros, self::arrayErrorExportParticularities($novaParticularidade, '', false));
                            continue;
                        }
                        if (!empty($this->service->validateBrandExists($novaParticularidade))) {
                            continue;
                        }
                    } else if ($novaParticularidade['CDTIPPART'] <= 6) {
                        $validaParticularidade = $this->service->getExistingProviderParticularities($novaParticularidade);
                        if (!empty($this->service->validateProviderExists($novaParticularidade))) {
                            continue;
                        }
                    } else {
                        $validaParticularidade = $this->service->getExistingBrandProviderParticularities($novaParticularidade);
                        if (!empty($this->service->validateBrandProviderExists($novaParticularidade))) {
                            continue;
                        }
                    }
                    if (!empty($validaParticularidade)) {
                        array_push($erros, self::arrayErrorExportParticularities($novaParticularidade, $validaParticularidade[0]));
                        continue;
                    }

                    $this->service->insertNewProductOrParticularity($novaParticularidade, $validaParticularidade);
                }
            }

            if (!empty($erros)) {
                $response->addDataSet(new DataSet("errors", $erros));
                throw new \Exception("sheetErrors", 1);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }

    public function arrayErrorExportParticularities($novaParticularidade, $validaParticularidade, $marcaValida = true)
    {
        $array['CDFILIAL']   = $novaParticularidade['CDFILIAL'];
        $array['NMFILIAL']   = $novaParticularidade['NMFILIAL'];
        $array['CDPRODUTO']  = $novaParticularidade['CDPRODUTO'];
        $array['NMPRODUTO']  = $novaParticularidade['NMPRODUTO'];
        $array['CDMARCA']    = $novaParticularidade['CDMARCA'];
        $array['NMMARCA']    = self::buscaMarca($novaParticularidade);
        $array['CDFORNECED'] = $novaParticularidade['CDFORNECED'];
        $array['NMFANTFORN'] = self::buscaFornecedor($novaParticularidade);
        $array['CDTIPPART']  = $novaParticularidade['CDTIPPART'];
        $array['TIPO']       = self::TIPOS[$novaParticularidade['CDTIPPART']];
        if ($marcaValida) {
            if ($novaParticularidade['CDTIPPART'] != $validaParticularidade['CDTIPPART']) {
                $array['MENSAGEM'] = self::MESSAGESEXPORT[$validaParticularidade['CDTIPPART']];
            } else {
                $array['MENSAGEM'] = self::MESSAGESEXPORTSAMETYPE[$validaParticularidade['CDTIPPART']];
            }
        } else {
            $array['MENSAGEM'] = "PesqMarcaProd";
        }

        return $array;
    }

    public function arrayNovaParticularidade($particularidade, $filial, $request)
    {
        $array['CDPRODUTO']  = $particularidade['CDPRODUTO'];
        $array['NMPRODUTO']  = $particularidade['NMPRODUTO'];
        $array['CDMARCA']    = $particularidade['CDMARCA'];
        $array['CDTIPPART']  = $particularidade['CDTIPPART'];
        $array['CDFORNECED'] = $particularidade['CDFORNECED'];
        $array['CDFILIAL']   = $filial['CDFILIAL'];
        $array['NMFILIAL']   = $filial['NMFILIAL'];
        $array['NRSEQPART']  = self::generateCode($array, $request);

        return $array;
    }

    public function arrayProdutoExcluido($particularidade, $filial)
    {
        $array['CDPRODUTO']  = $particularidade['CDPRODUTO'];
        $array['CDFILIAL']   = $filial['CDFILIAL'];

        return $array;
    }

    public function buscaMarca($row)
    {
        if ($row['CDMARCA'] != null) {
            $marca = $this->service->buscaMarca($row);
            return $marca[0]['NMMARCA'];
        } else {
            return null;
        }
    }

    public function buscaFornecedor($row)
    {
        if ($row['CDFORNECED'] != null) {
            $fornecedor = $this->service->buscaFornecedor($row);
            return $fornecedor[0]['NMFANTFORN'];
        } else {
            return null;
        }
    }

    public function buscaFiliais($row)
    {
        $filiais = array();

        if ($row['CDFILIAL'] != "T") {
            $unidades = $this->service->buscaFilial($row);
            $filiais = array_merge($filiais, $unidades);
        }
        if (isset($row['CDGRUPFILI'])) {
            $grupoFiliais = $this->service->buscaGrupoFilial($row);
            $filiais = array_merge($filiais, $grupoFiliais);
        }

        return $filiais;
    }

    public function buscaParticularidades($row)
    {
        $row['CDPRODFIN'] = str_pad($row['CDPRODFIN'], 20, 'Z', STR_PAD_RIGHT);
        $particularidades = $this->service->buscaParticularidade($row);
        if (count($particularidades) == 0) {
            throw new \Exception("NoParticularitiesFoundExported");
        }
        return $particularidades;
    }

    public function particularitiesSelect(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        $validation = $this->service->particularitiesSelect($row);
        $response->addDataSet(new DataSet("resp", $validation));
    }
}
