<?php
require_once 'base/GpesitufuncmEntityBase.php';


/**
 * GpesitufuncmEntity.
 *
 * @package Entity
 *
 * @author  alessandrobartels
 */
class GpesitufuncmEntity extends GpeSitufuncmEntityBase
{

    const SITUACAO_FUNCIONAL_NORMAL = 1;
    const SITUACAO_FUNCIONAL_MATERNIDADE = 5;/*BORBA 01/12/2014 Para atender a solicitação 153873*/
    const SITUACAO_FUNCIONAL_ATESTADO = 12; /*BORBA 21/09/2014 Para atender a solicitação 155577*/
    const SITUACAO_FUNCIONAL_DOENCA_MENOR_15 = 15; /*BORBA 10/02/2015 Para atender a solicitação 154336*/
    const SITUACAO_FUNCIONAL_DOENCA_MAIOR_15 = 6; /*BORBA 10/02/2015 Para atender a solicitação 154336*/

    private $_situFuncResumo = array();
    public $historico;

    /**
     * Array que armazena os tipos de situacoes funcionais situacoes funcionais.
     *
     * @return array
     *
     * @author alessandrobartels - 09/03/2011
     */
    public function getSituacaoFuncionalResumo(){
        return $this->_situFuncResumo;
    }


    /**
     * Adiciona itens GpesitufuncmEntity na lista.
     *
     * @param integer $nrOrgPadrao   Numero da Organizacao Padrao.
     * @param integer $nrOrgTrabalho Numero da Organizacao de Trabalho.
     *
     * @return void
     *
     * @author alessandrobartels
     *         alessandrobartels - 04/07/2011 - Parametrizacao de $nrOrgPadrao e $nrOrgTrabalho
     */
    private function _inicializaLista($nrOrgPadrao, $nrOrgTrabalho){
        $tmp = Gpesitufuncm::retornaSituacaoFuncional($nrOrgPadrao, $nrOrgTrabalho);

        foreach ($tmp as $tmp_situacao) {
            $this->addItem($tmp_situacao, $tmp_situacao->getNrsitufuncm());

            // Inicializa array de Situacao Funcional Resumo.
            $this->_situFuncResumo[$tmp_situacao->getNrsitufuncm()] = 0;
        }
    }


    /**
     * Retorna da lista instancia de GpesitufuncmEntity.
     *
     * @param integer $nrOrgPadrao   Numero da Organizacao Padrao.
     * @param integer $nrOrgTrabalho Numero da Organizacao de Trabalho.
     * @param integer $nr_situacao   Numero da Situacao Funcional.
     *
     * @return GpesitufuncmEntity
     *
     * @author alessandrobartels
     *         alessandrobartels - 04/07/2011 - Parametrizacao de $nrOrgPadrao e $nrOrgTrabalho
     */
    public function retornaSituacaoFuncional($nrOrgPadrao, $nrOrgTrabalho, $nr_situacao){
        if ($nr_situacao > 0) {
            $itens = $this->getItems();

            if (count($itens) === 0) {
                $this->_inicializaLista($nrOrgPadrao, $nrOrgTrabalho);
                $itens = $this->getItems();
            }

            return $itens[$nr_situacao];
        }

    }


    /**
     * Cria a Situacao Funcional Mestre e retorna seu numero
     *
     * @param $dtVigencia TkDateTime Data de Inicio de vigencia
     * @return integer
     *
     * @author alessandrobartels - 21/12/2012
     */
    public function criaSituacaoFuncional($dtVigencia){

        $situacaoFuncional = new self();
        $situacaoFuncional->setDtinivigencia($dtVigencia);
        $situacaoFuncional->persist();

        $nrSituacao = $situacaoFuncional->getNrsitufuncm();

        unset($situacaoFuncional);
        return $nrSituacao;
    }


    /**
     * Enter description here ...
     *
     * @param integer $nrsitufuncm Nr Situa��o Funcional.
     * @param integer $nrorgTrab   Nr Org.
     *
     * @return boolean
     *
     * @author luistavares 11/08/2011
     */
    public function novoMestre($nrsitufuncm, $nrorgTrab){
        try{
            $now = new TkDateTime();
            $now = $now->toString(TkDateTime::FORMAT_DATE);

            $this->setNrsitufuncm($nrsitufuncm);
            $this->setDtinivigencia($now);
            $this->setNrorg($nrorgTrab);
            $this->setDtinclusao($now);

            $this->persist();

            return true;
        }
        catch (Exception $e){
            return false;
        }

    }


    /**
     * Enter description here ...
     *
     * @param integer $nrsitufuncm Nr Situa��o Funcional.
     * @param integer $nrorgTrab   Nr Org.
     *
     * @return boolean
     *
     * @author luistavares 11/08/2011
     */
    public function destroiMestre($nrsitufuncm, $nrorgTrab){
        try{
            $situfuncm = Gpesitufuncm::retornaSituFuncPelaChave($nrsitufuncm, $nrorgTrab);
            if (is_object($situfuncm) === true) {
                /* Adiciona um item � lista, ap�s isso, o item � removido da lista
                 * e enviado para lista de itens removidos, lista essa que ir� apagar
                 * todos seus itens quando executar o persist(). */
                $this->addItem($situfuncm, $nrsitufuncm);
                $this->removeItem($situfuncm);
                $this->persist();

                return true;
            } else {
                return false;
            }
        }
        catch (Exception $e){
            tk_warning($e->getMessage());
            return false;
        }

    }

    /**
     * @return \GpesitufunchEntity
     */
    public function getHistorico() {
        if (!$this->historico) {
            $this->historico = \Gpesitufunch::getObjectByWhere(
                'NRSITUFUNCM = ?',
                array($this->getNrsitufuncm())
            );
        }
        return $this->historico;
    }

}

